@@ -46,18 +46,19 @@ class SquareletClient:
         self.session = requests.Session()
         self.access_token = None
         self.refresh_token = None
+        self._user_id = None
         self._set_tokens()
 
         # Apply rate limiting
         if rate_limit:
             # Apply rate limit decorator
-            self._request = ratelimit.limits(calls=RATE_LIMIT, period=RATE_PERIOD)(
-                self._request
+            self.request = ratelimit.limits(calls=RATE_LIMIT, period=RATE_PERIOD)(
+                self.request
             )
 
             # Apply sleep_and_retry if rate_limit_sleep is enabled
             if rate_limit_sleep:
-                self._request = ratelimit.sleep_and_retry(self._request)
+                self.request = ratelimit.sleep_and_retry(self.request)
 
     def _set_tokens(self):
         """Set the refresh and access tokens"""
@@ -69,11 +70,9 @@ class SquareletClient:
             self.access_token, self.refresh_token = self._get_tokens(
                 self.username, self.password
             )
-        elif self.access_token:
-            pass  # Already have access token, do nothing
         else:
-            raise ValueError("No tokens found")
-
+            self.access_token = None
+            self.refresh_token = None
         if self.access_token:
             self.session.headers.update(
                 {"Authorization": f"Bearer {self.access_token}"}
@@ -98,11 +97,21 @@ class SquareletClient:
     def _refresh_tokens(self, refresh_token):
         """Refresh the access and refresh tokens"""
         response = self.requests_retry_session().post(
-            f"{self.auth_uri}/refresh/", json={"refresh": refresh_token}
+            f"{self.auth_uri}refresh/",
+            json={"refresh": refresh_token},
+            timeout=self.timeout,
         )
-        return response["access"], response["refresh"]
 
-    def _request(self, method, url, raise_error=True, **kwargs):
+        if response.status_code == 401:
+            # refresh token is expired
+            return self._get_tokens(self.username, self.password)
+
+        self.raise_for_status(response)
+
+        json = response.json()
+        return (json["access"], json["refresh"])
+
+    def request(self, method, url, raise_error=True, **kwargs):
         """Generic method to make API requests"""
         # pylint: disable=method-hidden
         logger.info("request: %s - %s - %s", method, url, kwargs)
@@ -128,7 +137,7 @@ class SquareletClient:
         if response.status_code in [403, 429] and set_tokens:
             self._set_tokens()  # Refresh tokens
             kwargs["set_tokens"] = False  # Prevent infinite loop
-            return self._request(
+            return self.request(
                 method, url, full_url=True, **kwargs
             )  # Retry the request
 
@@ -166,7 +175,7 @@ class SquareletClient:
         """Generate methods for each HTTP request type (GET, POST, etc.)"""
         methods = ["get", "post", "put", "delete", "patch", "head", "options"]
         if attr in methods:
-            return partial(self._request, attr)
+            return partial(self.request, attr)
         raise AttributeError(
             f"'{self.__class__.__name__}' object has no attribute '{attr}'"
         )
@@ -203,3 +212,12 @@ class SquareletClient:
             if exc.response.status_code == 401:
                 raise CredentialsFailedError(response=exc.response) from exc
             raise APIError(response=exc.response) from exc
+
+    @property
+    def user_id(self):
+        """Returns the user ID of the user"""
+        if self._user_id is None:
+            user_data = self.request("get", "users/me/").json()
+            user_id = user_data["id"]
+            self._user_id = user_id
+        return self._user_id
